@@ -28,17 +28,25 @@ def run(args, cwd = None, shell = False, kill_tree = True, timeout = -1,
         if timeout != -1:
             alarm(0)
     except Alarm:
-        pids = [p.pid]
-        if kill_tree:
-            pids.extend(get_process_children(p.pid))
-        for pid in pids:
-            kill(pid, SIGKILL)
-        return -9, '', ''
+        return kill_process(p.pid, kill_tree)
     return p.returncode, stdout, stderr
 
+def kill_process(pid, recursively):
+    pids = [pid]
+    if recursively:
+        pids.extend(get_process_children(pid))
+
+    for p in pids:
+        kill(p, SIGKILL)
+
+    return -9, '', ''
 
 def get_process_children(pid):
     p = Popen('ps --no-headers -o pid --ppid %d' % pid, shell = True,
               stdout = PIPE, stderr = PIPE)
     stdout, _stderr = p.communicate()
-    return [int(p) for p in stdout.split()]
+    result = [int(p) for p in stdout.split()]
+    for child in result[:]:
+        result.extend(get_process_children(child))
+    return result
+
